@@ -30,6 +30,13 @@
  * $FreeBSD: head/sys/sys/queue.h 221843 2011-05-13 15:49:23Z mdf $
  */
 
+#if defined(__NetBSD__)
+/* NetBSD and FreeBSD implementations differ in details.
+ * pullin the native one otherwise system headers won't build */
+#include <sys/queue.h>
+
+#else
+
 #ifndef _SYS_QUEUE_H_
 #define _SYS_QUEUE_H_
 
@@ -135,10 +142,6 @@ struct qm_trace {
 #define TRASHIT(x)
 #endif  /* QUEUE_MACRO_DEBUG */
 
-/* Disabled for tup since we don't use it and it conflicts with definitions in
- * winnt.h for Windows
- */
-#if 0
 /*
  * Singly-linked List declarations.
  */
@@ -221,7 +224,6 @@ struct {                                                                \
         SLIST_FIRST(head1) = SLIST_FIRST(head2);                        \
         SLIST_FIRST(head2) = swap_first;                                \
 } while (0)
-#endif
 
 /*
  * Singly-linked Tail queue declarations.
@@ -638,3 +640,5 @@ struct {                                                                \
 } while (0)
 
 #endif /* !_SYS_QUEUE_H_ */
+
+#endif /* defined(__NetBSD__) */
