@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -123,10 +123,6 @@ tupid_t tup_file_mod_mtime(tupid_t dt, const char *file, time_t mtime,
 		log_debug_tent("Create", tent, ", mtime=%li\n", mtime);
 		new = 1;
 	} else {
-		/* Always ignore generated files when it's a .gitignore file, since that happens during parsing. */
-		if(strcmp(file, ".gitignore") == 0)
-			ignore_generated = 1;
-
 		/* If we are ignoring generated files (ie: from the monitor when it catches
 		 * an event from the updater creating output files), then disable force.
 		 * In this case we only want to mark the generated files again if the user
@@ -444,6 +440,13 @@ int tup_del_id_type(tupid_t tupid, enum TUP_NODE_TYPE type, int force, int *modi
 		if(tup_db_unflag_modify(tupid) < 0)
 			return -1;
 
+		/* Transient files don't need a warning, since tup likely was
+		 * the one who deleted them.
+		 */
+		if(is_transient_tent(tent)) {
+			return 0;
+		}
+
 		/* Only display a warning if the command isn't already in the
 		 * modify list. It's possible that the command hasn't actually
 		 * been executed yet.
