@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -113,12 +113,15 @@ int tup_db_maybe_add_create_list(tupid_t tupid);
 int tup_db_add_create_list(tupid_t tupid);
 int tup_db_add_modify_list(tupid_t tupid);
 int tup_db_add_variant_list(tupid_t tupid);
+int tup_db_add_transient_list(tupid_t tupid);
 int tup_db_in_create_list(tupid_t tupid);
 int tup_db_in_modify_list(tupid_t tupid);
+int tup_db_in_transient_list(tupid_t tupid);
 int tup_db_unflag_config(tupid_t tupid);
 int tup_db_unflag_create(tupid_t tupid);
 int tup_db_unflag_modify(tupid_t tupid);
 int tup_db_unflag_variant(tupid_t tupid);
+int tup_db_unflag_transient(tupid_t tupid);
 
 /* Link operations */
 int tup_db_create_link(tupid_t a, tupid_t b, int style);
@@ -127,7 +130,8 @@ int tup_db_link_exists(tupid_t a, tupid_t b, int style,
 		       int *exists);
 int tup_db_get_incoming_link(struct tup_entry *tent, struct tup_entry **incoming);
 int tup_db_delete_links(tupid_t tupid);
-int tup_db_write_outputs(FILE *f, tupid_t cmdid, struct tent_entries *root,
+int tup_db_write_outputs(FILE *f, struct tup_entry *cmdtent,
+			 struct tent_entries *root,
 			 struct tent_entries *exclusion_root,
 			 struct tup_entry *group,
 			 struct tup_entry **old_group,
@@ -167,7 +171,7 @@ int tup_db_config_get_int(const char *lval, int def, int *result);
 /* Var operations */
 int tup_db_set_var(tupid_t tupid, const char *value);
 struct tup_entry *tup_db_get_var(struct variant *variant, const char *var, int varlen, struct estring *e);
-int tup_db_read_vars(int root_fd, struct tup_entry *dtent, const char *file,
+int tup_db_read_vars(struct tup_entry *dtent, const char *file,
 		     struct tup_entry *var_dtent, const char *vardict_file);
 int tup_db_delete_tup_config(struct tup_entry *tent);
 
@@ -184,9 +188,9 @@ int is_virtual_tent(struct tup_entry *tent);
 tupid_t exclusion_dt(void);
 
 /* Tree operations */
-int tup_db_dirtype(tupid_t dt, struct tent_list_head *head, struct tent_entries *root, int *count, enum TUP_NODE_TYPE type);
-int tup_db_srcid_to_tree(tupid_t srcid, struct tent_entries *root, int *count, enum TUP_NODE_TYPE type);
-int tup_db_type_to_tree(struct tent_entries *root, int *count, enum TUP_NODE_TYPE type);
+int tup_db_dirtype(tupid_t dt, struct tent_list_head *head, struct tent_entries *root, enum TUP_NODE_TYPE type);
+int tup_db_srcid_to_tree(tupid_t srcid, struct tent_entries *root, enum TUP_NODE_TYPE type);
+int tup_db_type_to_tree(struct tent_entries *root, enum TUP_NODE_TYPE type);
 int tup_db_is_generated_dir(tupid_t dt);
 
 /* scanner operations */
