@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2009-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2009-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -459,8 +459,8 @@ static struct tup_entry *new_entry(tupid_t tupid, tupid_t dt,
 	tent->mtime = mtime;
 	tent->srcid = srcid;
 	tent->variant = NULL;
-	RB_INIT(&tent->stickies);
-	RB_INIT(&tent->group_stickies);
+	tent_tree_init(&tent->stickies);
+	tent_tree_init(&tent->group_stickies);
 	tent->retrieved_stickies = 0;
 	tent->incoming = NULL;
 	tent->refcount = 0;
@@ -575,7 +575,7 @@ int tup_entry_clear(void)
 	return 0;
 }
 
-int tup_entry_add_ghost_tree(struct tup_entry *tent, struct tent_entries *root)
+int tup_entry_add_ghost_tree(struct tent_entries *root, struct tup_entry *tent)
 {
 	if(tent->type == TUP_NODE_GHOST || tent->type == TUP_NODE_GROUP ||
 	   tent->type == TUP_NODE_GENERATED_DIR) {
@@ -593,7 +593,7 @@ int tup_entry_debug_add_all_ghosts(struct tent_entries *root)
 		struct tup_entry *tent;
 
 		tent = container_of(tt, struct tup_entry, tnode);
-		if(tup_entry_add_ghost_tree(tent, root) < 0)
+		if(tup_entry_add_ghost_tree(root, tent) < 0)
 			return -1;
 	}
 	return 0;
@@ -754,6 +754,11 @@ int get_relative_dir(FILE *f, struct estring *e, tupid_t start, tupid_t end)
 	return 0;
 }
 
+int is_transient_tent(struct tup_entry *tent)
+{
+	return memchr(tent->flags, 't', tent->flagslen) != NULL;
+}
+
 int exclusion_match(FILE *f, struct tent_entries *exclusion_root, const char *s, int *match)
 {
 	struct tent_tree *tt;
