@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -56,12 +56,12 @@ int init_file_info(struct file_info *info, const char *variant_dir, int do_unlin
 	TAILQ_INIT(&info->var_list);
 	TAILQ_INIT(&info->mapping_list);
 	TAILQ_INIT(&info->tmpdir_list);
-	RB_INIT(&info->sticky_root);
-	RB_INIT(&info->normal_root);
-	RB_INIT(&info->group_sticky_root);
-	RB_INIT(&info->used_groups_root);
-	RB_INIT(&info->output_root);
-	RB_INIT(&info->exclusion_root);
+	tent_tree_init(&info->sticky_root);
+	tent_tree_init(&info->normal_root);
+	tent_tree_init(&info->group_sticky_root);
+	tent_tree_init(&info->used_groups_root);
+	tent_tree_init(&info->output_root);
+	tent_tree_init(&info->exclusion_root);
 	pthread_mutex_init(&info->lock, NULL);
 	pthread_cond_init(&info->cond, NULL);
 	/* Root variant gets a NULL variant_dir so we can skip trying to do the
@@ -98,6 +98,59 @@ void finfo_unlock(struct file_info *info)
 	pthread_mutex_unlock(&info->lock);
 }
 
+#ifndef _WIN32
+static int get_symlink(const char *filename, char **ret)
+{
+	char linkbuf[PATH_MAX];
+	ssize_t linklen;
+	linklen = readlinkat(tup_top_fd(), filename, linkbuf, sizeof(linkbuf));
+	if(linklen <= 0)
+		return 0;
+	linkbuf[linklen] = 0;
+	if(linkbuf[0] == '/') {
+		*ret = strdup(linkbuf);
+		if(!*ret) {
+			perror("strdup");
+			return -1;
+		}
+	} else {
+		char *last_slash;
+		int dirlen;
+		last_slash = strrchr(filename, '/');
+		if(!last_slash) {
+			*ret = malloc(linklen + 1);
+			strcpy(*ret, linkbuf);
+		} else {
+			dirlen = last_slash - filename + 1;
+			*ret = malloc(dirlen + linklen + 1);
+			if(!*ret) {
+				perror("malloc");
+				return -1;
+			}
+			strncpy(*ret, filename, dirlen);
+			strcpy(*ret + dirlen, linkbuf);
+		}
+	}
+	return 0;
+}
+
+static int add_symlinks(const char *path, struct file_info *finfo)
+{
+	char *linkpath = NULL;
+
+	if(get_symlink(path, &linkpath) < 0)
+		return -1;
+	if(linkpath) {
+		if(handle_open_file(ACCESS_READ, linkpath, finfo) < 0) {
+			fprintf(stderr, "tup error: Failed to call handle_open_file on a symlink event '%s'\n", linkpath);
+			return -1;
+		}
+		free(linkpath);
+	}
+	return 0;
+}
+#endif
+
 int handle_file_dtent(enum access_type at, struct tup_entry *dtent,
 		      const char *filename, struct file_info *info)
 {
@@ -136,6 +189,12 @@ int handle_file(enum access_type at, const char *filename, const char *file2,
 		rc = handle_rename(filename, file2, info);
 	} else {
 		rc = handle_open_file(at, filename, info);
+#ifndef _WIN32
+		if(at == ACCESS_READ) {
+			if(add_symlinks(filename, info) < 0)
+				return -1;
+		}
+#endif
 	}
 	finfo_unlock(info);
 
@@ -430,7 +489,7 @@ static int file_set_mtime(struct tup_entry *tent, const char *file)
 static int add_config_files_locked(struct file_info *finfo, struct tup_entry *tent, int full_deps)
 {
 	struct file_entry *r;
-	struct tent_entries root = {NULL};
+	struct tent_entries root = TENT_ENTRIES_INITIALIZER;
 
 	while(!TAILQ_EMPTY(&finfo->read_list)) {
 		r = TAILQ_FIRST(&finfo->read_list);
@@ -458,7 +517,7 @@ static int add_parser_files_locked(struct file_info *finfo,
 	struct file_entry *r;
 	struct mapping *map;
 	struct tup_entry *tent;
-	struct tent_entries tmproot = {NULL};
+	struct tent_entries tmproot = TENT_ENTRIES_INITIALIZER;
 	struct tent_tree *tt;
 	int map_bork = 0;
 
@@ -633,7 +692,7 @@ static int update_write_info(FILE *f, tupid_t cmdid, struct file_info *info,
 	struct file_entry *r;
 	struct file_entry *tmp;
 	struct tup_entry *tent;
-	struct tent_entries root = {NULL};
+	struct tent_entries root = TENT_ENTRIES_INITIALIZER;
 	int write_bork = 0;
 
 	while(!TAILQ_EMPTY(&info->write_list)) {
@@ -776,7 +835,7 @@ static int update_read_info(FILE *f, tupid_t cmdid, struct file_info *info,
 			    int *important_link_removed)
 {
 	struct file_entry *r;
-	struct tent_entries root = {NULL};
+	struct tent_entries root = TENT_ENTRIES_INITIALIZER;
 
 	while(!TAILQ_EMPTY(&info->read_list)) {
 		int match = 0;
