@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -62,22 +62,72 @@ struct node *create_node(struct graph *g, struct tup_entry *tent)
 	LIST_INIT(&n->incoming);
 	n->tnode.tupid = tent->tnode.tupid;
 	n->tent = tent;
+	n->active_list = NULL;
 	n->state = STATE_INITIALIZED;
 	n->already_used = 0;
 	n->expanded = 0;
 	n->parsing = 0;
 	n->marked = 0;
 	n->skip = 1;
-	TAILQ_INSERT_TAIL(&g->node_list, n, list);
+	n->counted = 0;
+	if(node_insert_tail(&g->node_list, n) < 0)
+		return NULL;
+
+	/* The transient field in struct node is used for determining when to
+	 * remove a transient file from the filesystem. It doesn't correspond
+	 * to transient_list in the db.
+	 */
+	if(tent->type == TUP_NODE_GENERATED && is_transient_tent(tent)) {
+		n->transient = TRANSIENT_PROCESSING;
+	} else {
+		n->transient = TRANSIENT_NONE;
+	}
 
 	if(tupid_tree_insert(&g->node_root, &n->tnode) < 0)
 		return NULL;
 	return n;
 }
 
+int node_insert_tail(struct node_head *head, struct node *n)
+{
+	if(n->active_list != NULL) {
+		fprintf(stderr, "tup internal error: Node %s is already in a list.\n", n->tent->name.s);
+		return -1;
+	}
+	TAILQ_INSERT_TAIL(head, n, list);
+	n->active_list = head;
+	return 0;
+}
+
+int node_insert_head(struct node_head *head, struct node *n)
+{
+	if(n->active_list != NULL) {
+		fprintf(stderr, "tup internal error: Node %s is already in a list.\n", n->tent->name.s);
+		return -1;
+	}
+	TAILQ_INSERT_HEAD(head, n, list);
+	n->active_list = head;
+	return 0;
+}
+
+int node_remove_list(struct node_head *head, struct node *n)
+{
+	if(!n->active_list) {
+		fprintf(stderr, "tup internal error: Node %s is not on a list.\n", n->tent->name.s);
+		return -1;
+	}
+	if(n->active_list != head) {
+		fprintf(stderr, "tup internal error: Node %s being removed from a different list.\n", n->tent->name.s);
+		return -1;
+	}
+	TAILQ_REMOVE(head, n, list);
+	n->active_list = NULL;
+	return 0;
+}
+
 static void remove_node_internal(struct graph *g, struct node *n)
 {
-	TAILQ_REMOVE(&g->node_list, n, list);
+	node_remove_list(n->active_list, n);
 	while(!LIST_EMPTY(&n->edges)) {
 		remove_edge(LIST_FIRST(&n->edges));
 	}
@@ -169,7 +219,7 @@ void remove_edge(struct edge *e)
 	mempool_free(&edge_pool, e);
 }
 
-int create_graph(struct graph *g, enum TUP_NODE_TYPE count_flags, enum TUP_NODE_TYPE count_flags2)
+int create_graph(struct graph *g, enum TUP_NODE_TYPE count_flags)
 {
 	root_entry.tnode.tupid = 0;
 	root_entry.dt = 0;
@@ -183,14 +233,13 @@ int create_graph(struct graph *g, enum TUP_NODE_TYPE count_flags, enum TUP_NODE_
 	TAILQ_INIT(&g->node_list);
 	TAILQ_INIT(&g->plist);
 	TAILQ_INIT(&g->removing_list);
-	RB_INIT(&g->gen_delete_root);
-	RB_INIT(&g->save_root);
-	g->gen_delete_count = 0;
-	RB_INIT(&g->cmd_delete_root);
-	g->cmd_delete_count = 0;
+	tent_tree_init(&g->transient_root);
+	tent_tree_init(&g->gen_delete_root);
+	tent_tree_init(&g->save_root);
+	tent_tree_init(&g->cmd_delete_root);
 
-	RB_INIT(&g->normal_dir_root);
-	RB_INIT(&g->parse_gitignore_root);
+	tent_tree_init(&g->normal_dir_root);
+	tent_tree_init(&g->parse_gitignore_root);
 	RB_INIT(&g->node_root);
 
 	g->cur = g->root = create_node(g, &root_entry);
@@ -198,7 +247,6 @@ int create_graph(struct graph *g, enum TUP_NODE_TYPE count_flags, enum TUP_NODE_
 		return -1;
 	g->num_nodes = 0;
 	g->count_flags = count_flags;
-	g->count_flags2 = count_flags2;
 	g->total_mtime = 0;
 	if(count_flags == TUP_NODE_GROUP)
 		g->style = TUP_LINK_GROUP;
@@ -227,26 +275,19 @@ void save_graphs(struct graph *g)
 	save_graph(stderr, g, ".tup/tmp/graph-trimmed-%i.dot");
 }
 
-static void expand_node(struct graph *g, struct node *n)
+static int expand_node(struct graph *g, struct node *n)
 {
 	n->expanded = 1;
-	TAILQ_REMOVE(&g->node_list, n, list);
-	TAILQ_INSERT_HEAD(&g->plist, n, list);
+	if(node_remove_list(&g->node_list, n) < 0)
+		return -1;
+	if(node_insert_head(&g->plist, n) < 0)
+		return -1;
+	return 0;
 }
 
-int build_graph_cb(void *arg, struct tup_entry *tent)
+static int make_edge(struct graph *g, struct node *n)
 {
-	struct graph *g = arg;
-	struct node *n;
-
-	n = find_node(g, tent->tnode.tupid);
-	if(n != NULL)
-		goto edge_create;
-	n = create_node(g, tent);
-	if(!n)
-		return -1;
-
-edge_create:
+	struct tup_entry *tent = n->tent;
 	if(n->state == STATE_PROCESSING) {
 		/* A circular dependency is not guaranteed to trigger this,
 		 * but it is easy to check before going through the graph.
@@ -259,7 +300,8 @@ edge_create:
 	}
 	if(n->expanded == 0) {
 		/* TUP_NODE_ROOT means we count everything */
-		if(n->tent->type == g->count_flags || n->tent->type == g->count_flags2 || g->count_flags == TUP_NODE_ROOT) {
+		if(n->tent->type == g->count_flags || g->count_flags == TUP_NODE_ROOT) {
+			n->counted = 1;
 			g->num_nodes++;
 			if(g->total_mtime != -1) {
 				if(n->tent->mtime == -1)
@@ -279,6 +321,75 @@ edge_create:
 	return 0;
 }
 
+static int attach_transient_cb(void *arg, struct tup_entry *tent)
+{
+	/* Transient nodes that don't otherwise need to be built are only added
+	 * if they can attach to something else in the DAG.
+	 */
+	struct graph *g = arg;
+	struct node *n;
+	n = find_node(g, tent->tnode.tupid);
+	if(n) {
+		return make_edge(g, n);
+	}
+	return 0;
+}
+
+int build_graph_transient_cb(void *arg, struct tup_entry *tent)
+{
+	struct graph *g = arg;
+
+	if(tent->type == TUP_NODE_CMD) {
+		return build_graph_cb(g, tent);
+	} else {
+		struct node *n;
+		n = find_node(g, tent->tnode.tupid);
+		if(n == NULL) {
+			n = create_node(g, tent);
+			if(!n)
+				return -1;
+			if(make_edge(g, n) < 0)
+				return -1;
+			if(node_remove_list(&g->plist, n) < 0)
+				return -1;
+			if(node_insert_tail(&g->node_list, n) < 0)
+				return -1;
+			n->state = STATE_FINISHED;
+		}
+	}
+	return 0;
+}
+
+int build_graph_non_transient_cb(void *arg, struct tup_entry *tent)
+{
+	/* Only build out nodes that aren't transient. Any transient nodes get
+	 * saved in g->transient_root for later processing.
+	 */
+	struct graph *g = arg;
+	if(tent->type == TUP_NODE_CMD && is_transient_tent(tent)) {
+		if(tent_tree_add(&g->transient_root, tent) < 0)
+			return -1;
+		return 0;
+	}
+
+	return build_graph_cb(arg, tent);
+}
+
+int build_graph_cb(void *arg, struct tup_entry *tent)
+{
+	struct graph *g = arg;
+	struct node *n;
+
+	n = find_node(g, tent->tnode.tupid);
+	if(n == NULL) {
+		n = create_node(g, tent);
+		if(!n)
+			return -1;
+	}
+
+	return make_edge(g, n);
+}
+
 static struct node *find_or_create_node(struct graph *g, struct tup_entry *tent)
 {
 	struct node *n;
@@ -309,7 +420,8 @@ int build_graph_group_cb(void *arg, struct tup_entry *tent)
 		return -1;
 
 	if(n->expanded == 0) {
-		expand_node(g, n);
+		if(expand_node(g, n) < 0)
+			return -1;
 	}
 
 	if(create_edge(g->cur, n, TUP_LINK_NORMAL) < 0)
@@ -337,10 +449,12 @@ static int build_group_cb(void *arg, struct tup_entry *tent,
 		return -1;
 
 	if(n->expanded == 0) {
-		expand_node(g, n);
+		if(expand_node(g, n) < 0)
+			return -1;
 	}
 	if(cmdn->expanded == 0) {
-		expand_node(g, cmdn);
+		if(expand_node(g, cmdn) < 0)
+			return -1;
 	}
 
 	if(create_edge(g->cur, cmdn, TUP_LINK_NORMAL) < 0)
@@ -351,10 +465,82 @@ static int build_group_cb(void *arg, struct tup_entry *tent,
 	return 0;
 }
 
+static int attach_transient_nodes(struct graph *g)
+{
+	struct tent_tree *tt;
+	struct tent_tree *tmp;
+	int rc = 0;
+	RB_FOREACH_SAFE(tt, tent_entries, &g->transient_root, tmp) {
+		struct tup_entry *tent;
+		struct node *cmdnode;
+		struct node *n;
+		struct edge *e;
+		int keep_cmd = 0;
+		tent = tt->tent;
+
+		if(find_node(g, tent->tnode.tupid)) {
+			continue;
+		}
+
+		/* Expand the command and its output files */
+		g->cur = create_node(g, tent);
+		cmdnode = g->cur;
+		if(tup_db_select_node_by_link(build_graph_cb, g, g->cur->tnode.tupid) < 0)
+			return -1;
+		LIST_FOREACH(e, &cmdnode->edges, list) {
+			g->cur = e->dest;
+			if(tup_db_select_node_by_link(attach_transient_cb, g, g->cur->tnode.tupid) < 0)
+				return -1;
+
+			/* We only need to keep the command if:
+			 *  1) The file doesn't already exist from a previous
+			 *     incomplete build (in the transient list)
+			 *   and
+			 *  2) We link to something in the DAG (!LIST_EMPTY)
+			 */
+			if(!tup_db_in_transient_list(g->cur->tent->tnode.tupid) && !LIST_EMPTY(&g->cur->edges)) {
+				keep_cmd = 1;
+				break;
+			}
+		}
+		if(keep_cmd) {
+			rc = 1;
+			LIST_FOREACH(e, &cmdnode->edges, list) {
+				n = e->dest;
+				if(node_remove_list(&g->plist, n) < 0)
+					return -1;
+				if(node_insert_tail(&g->node_list, n) < 0)
+					return -1;
+				n->state = STATE_FINISHED;
+			}
+			g->cur = g->root;
+			if(make_edge(g, cmdnode) < 0)
+				return -1;
+			if(node_remove_list(&g->plist, cmdnode) < 0)
+				return -1;
+			if(node_insert_tail(&g->node_list, cmdnode) < 0)
+				return -1;
+			tent_tree_rm(&g->transient_root, tt);
+		} else {
+			struct edge *tmpe;
+			LIST_FOREACH_SAFE(e, &cmdnode->edges, list, tmpe) {
+				if(!tup_db_in_transient_list(e->dest->tent->tnode.tupid)) {
+					remove_node_internal(g, e->dest);
+				}
+			}
+			remove_node_internal(g, cmdnode);
+		}
+	}
+	return rc;
+}
+
 int build_graph(struct graph *g)
 {
 	struct node *cur;
 
+	/* First fill out the graph with everything that we know needs to be
+	 * built.
+	 */
 	while(!TAILQ_EMPTY(&g->plist)) {
 		cur = TAILQ_FIRST(&g->plist);
 		if(cur->state == STATE_INITIALIZED) {
@@ -374,8 +560,10 @@ int build_graph(struct graph *g)
 			cur->state = STATE_PROCESSING;
 		} else if(cur->state == STATE_PROCESSING) {
 			DEBUGP("remove node from stack: %lli\n", cur->tnode.tupid);
-			TAILQ_REMOVE(&g->plist, cur, list);
-			TAILQ_INSERT_TAIL(&g->node_list, cur, list);
+			if(node_remove_list(&g->plist, cur) < 0)
+				return -1;
+			if(node_insert_tail(&g->node_list, cur) < 0)
+				return -1;
 			cur->state = STATE_FINISHED;
 		} else if(cur->state == STATE_FINISHED) {
 			fprintf(stderr, "tup internal error: STATE_FINISHED node %lli in plist\n", cur->tnode.tupid);
@@ -384,10 +572,35 @@ int build_graph(struct graph *g)
 		}
 	}
 
+	/* Second, attach any commands that build transient files that are
+	 * currently missing. This has to be done recursively until no nodes
+	 * are added in case there are chains of transient nodes.
+	 */
+	while(1) {
+		int rc;
+		rc = attach_transient_nodes(g);
+		if(rc < 0)
+			return -1;
+		if(rc == 0)
+			break;
+	}
+	/* Anything remaining in transient_root are nodes that we don't need to
+	 * rebuild.
+	 */
+	free_tent_tree(&g->transient_root);
+
 	if(g->style != TUP_LINK_GROUP)
 		if(add_graph_stickies(g) < 0)
 			return -1;
 
+	if(!TAILQ_EMPTY(&g->plist)) {
+		struct node *tmpn;
+		fprintf(stderr, "tup internal error: plist should be empty after graph building.\n");
+		TAILQ_FOREACH(tmpn, &g->plist, list) {
+			printf(" - %s\n", tmpn->tent->name.s);
+		}
+		return -1;
+	}
 	return 0;
 }
 
@@ -404,7 +617,7 @@ int add_graph_stickies(struct graph *g)
 
 	TAILQ_FOREACH(n, &g->node_list, list) {
 		if(n->tent->type == TUP_NODE_CMD) {
-			struct tent_entries sticky_root = {NULL};
+			struct tent_entries sticky_root = TENT_ENTRIES_INITIALIZER;
 			struct tent_tree *tt;
 			struct node *inputn;
 
@@ -429,13 +642,12 @@ static int add_file_cb(void *arg, struct tup_entry *tent)
 	struct node *n;
 
 	n = find_node(g, tent->tnode.tupid);
-	if(n != NULL)
-		goto edge_create;
-	n = create_node(g, tent);
-	if(!n)
-		return -1;
+	if(n == NULL) {
+		n = create_node(g, tent);
+		if(!n)
+			return -1;
+	}
 
-edge_create:
 	if(n->state == STATE_PROCESSING) {
 		/* A circular dependency is not guaranteed to trigger this,
 		 * but it is easy to check before going through the graph.
@@ -446,11 +658,11 @@ edge_create:
 		return -1;
 	}
 	if(n->expanded == 0) {
-		if(n->tent->type == g->count_flags)
-			g->num_nodes++;
 		n->expanded = 1;
-		TAILQ_REMOVE(&g->node_list, n, list);
-		TAILQ_INSERT_HEAD(&g->plist, n, list);
+		if(node_remove_list(&g->node_list, n) < 0)
+			return -1;
+		if(node_insert_head(&g->plist, n) < 0)
+			return -1;
 	}
 
 	if(create_edge(g->cur, n, TUP_LINK_NORMAL) < 0)
@@ -464,7 +676,7 @@ int nodes_are_connected(struct tup_entry *src, struct tent_entries *valid_root,
 	struct graph g;
 	struct node *n;
 
-	if(create_graph(&g, TUP_NODE_CMD, -1) < 0)
+	if(create_graph(&g, TUP_NODE_CMD) < 0)
 		return -1;
 	n = create_node(&g, src);
 	if(!n)
@@ -472,8 +684,10 @@ int nodes_are_connected(struct tup_entry *src, struct tent_entries *valid_root,
 	if(create_edge(g.cur, n, TUP_LINK_NORMAL) < 0)
 		return -1;
 	n->expanded = 1;
-	TAILQ_REMOVE(&g.node_list, n, list);
-	TAILQ_INSERT_HEAD(&g.plist, n, list);
+	if(node_remove_list(&g.node_list, n) < 0)
+		return -1;
+	if(node_insert_head(&g.plist, n) < 0)
+		return -1;
 
 	*connected = 0;
 	while(!TAILQ_EMPTY(&g.plist)) {
@@ -493,8 +707,10 @@ int nodes_are_connected(struct tup_entry *src, struct tent_entries *valid_root,
 			n->state = STATE_PROCESSING;
 		} else if(n->state == STATE_PROCESSING) {
 			DEBUGP("remove node from stack: %lli\n", n->tnode.tupid);
-			TAILQ_REMOVE(&g.plist, n, list);
-			TAILQ_INSERT_TAIL(&g.node_list, n, list);
+			if(node_remove_list(&g.plist, n) < 0)
+				return -1;
+			if(node_insert_tail(&g.node_list, n) < 0)
+				return -1;
 			n->state = STATE_FINISHED;
 		} else if(n->state == STATE_FINISHED) {
 			fprintf(stderr, "tup internal error: STATE_FINISHED node %lli in plist\n", n->tnode.tupid);
@@ -550,7 +766,8 @@ static void mark_nodes(struct node *n)
 
 static int prune_node(struct graph *g, struct node *n, int *num_pruned, enum graph_prune_type gpt, int verbose)
 {
-	if(n->tent->type == g->count_flags && n->expanded) {
+	struct edge *e;
+	if(n->counted) {
 		if(n->tent->type != TUP_NODE_CMD) {
 			fprintf(stderr, "tup internal error: node of type %i trying to add to the modify list in prune_graph\n", n->tent->type);
 			return -1;
@@ -574,6 +791,14 @@ static int prune_node(struct graph *g, struct node *n, int *num_pruned, enum gra
 			printf("\n");
 		}
 	}
+	/* If a partial update prunes a node from the DAG, we want to still
+	 * keep any transient files we depend on around until they are actually
+	 * used.
+	 */
+	LIST_FOREACH(e, &n->incoming, destlist) {
+		if(e->src->transient)
+			e->src->transient = TRANSIENT_NONE;
+	}
 	/* Normal files are not pruned so we can make sure we update the mtime
 	 * in the tup database (t6079). Any dependent commands are flagged as
 	 * modify above, so they will still run later.
@@ -1154,7 +1379,7 @@ int add_group_circ_check(struct tup_entry *tent)
 {
 	struct node *n;
 	if(!group_graph_inited) {
-		if(create_graph(&group_graph, TUP_NODE_GROUP, -1) < 0)
+		if(create_graph(&group_graph, TUP_NODE_GROUP) < 0)
 			return -1;
 		group_graph_inited = 1;
 	}
@@ -1165,8 +1390,10 @@ int add_group_circ_check(struct tup_entry *tent)
 	if(!n) {
 		return -1;
 	}
-	TAILQ_REMOVE(&group_graph.node_list, n, list);
-	TAILQ_INSERT_HEAD(&group_graph.plist, n, list);
+	if(node_remove_list(&group_graph.node_list, n) < 0)
+		return -1;
+	if(node_insert_head(&group_graph.plist, n) < 0)
+		return -1;
 	n->expanded = 1;
 	return 0;
 }
