@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -45,6 +45,12 @@ struct tup_entry;
 #define STATE_FINISHED 2
 #define STATE_REMOVING 3
 
+enum transient_type {
+	TRANSIENT_NONE,
+	TRANSIENT_PROCESSING,
+	TRANSIENT_DELETE,
+};
+
 enum graph_prune_type {
 	GRAPH_PRUNE_GENERATED,
 	GRAPH_PRUNE_ALL,
@@ -56,6 +62,7 @@ struct node {
 	struct edge_head incoming;
 	struct tupid_tree tnode;
 	struct tup_entry *tent;
+	struct node_head *active_list;
 
 	unsigned char state;
 	unsigned char already_used;
@@ -63,6 +70,8 @@ struct node {
 	unsigned char parsing;
 	unsigned char marked;
 	unsigned char skip;
+	unsigned char counted;
+	unsigned char transient;
 };
 TAILQ_HEAD(node_head, node);
 
@@ -70,18 +79,16 @@ struct graph {
 	struct node_head node_list;
 	struct node_head plist;
 	struct node_head removing_list;
+	struct tent_entries transient_root;
 	struct node *root;
 	struct node *cur;
 	int num_nodes;
 	struct tupid_entries node_root;
 	enum TUP_NODE_TYPE count_flags;
-	enum TUP_NODE_TYPE count_flags2;
 	time_t total_mtime;
 	struct tent_entries gen_delete_root;
 	struct tent_entries save_root;
-	int gen_delete_count;
 	struct tent_entries cmd_delete_root;
-	int cmd_delete_count;
 	struct tent_entries normal_dir_root;
 	struct tent_entries parse_gitignore_root;
 	int style;
@@ -90,13 +97,18 @@ struct graph {
 struct node *find_node(struct graph *g, tupid_t tupid);
 struct node *create_node(struct graph *g, struct tup_entry *tent);
 void remove_node(struct graph *g, struct node *n);
+int node_insert_tail(struct node_head *head, struct node *n);
+int node_insert_head(struct node_head *head, struct node *n);
+int node_remove_list(struct node_head *head, struct node *n);
 
 int create_edge(struct node *n1, struct node *n2, int style);
 void remove_edge(struct edge *e);
 
-int create_graph(struct graph *g, enum TUP_NODE_TYPE count_flags, enum TUP_NODE_TYPE count_flags2);
+int create_graph(struct graph *g, enum TUP_NODE_TYPE count_flags);
 int destroy_graph(struct graph *g);
 void save_graphs(struct graph *g);
+int build_graph_transient_cb(void *arg, struct tup_entry *tent);
+int build_graph_non_transient_cb(void *arg, struct tup_entry *tent);
 int build_graph_cb(void *arg, struct tup_entry *tent);
 int build_graph_group_cb(void *arg, struct tup_entry *tent);
 int build_graph(struct graph *g);
