@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2011-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2011-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -123,6 +123,12 @@ void clear_progress(void)
 
 void tup_main_progress(const char *s)
 {
+	if(sum != total) {
+		fprintf(stderr, "tup internal error: previous progress bar sized incorrectly: %i, %i\n", sum, total);
+		if(getenv("TUP_VALGRIND")) {
+			exit(1);
+		}
+	}
 	clear_active(stdout);
 	cur_phase++;
 	tup_show_message(s);
@@ -198,6 +204,9 @@ void show_result(struct tup_entry *tent, int is_error, struct timespan *ts, cons
 	sum++;
 	if(sum > total) {
 		fprintf(stderr, "tup internal error: progress bar is sized incorrectly.\n");
+		if(getenv("TUP_VALGRIND")) {
+			exit(1);
+		}
 	}
 
 	if(quiet && !always_display)
@@ -308,7 +317,11 @@ void show_progress(int active, enum TUP_NODE_TYPE type)
 
 		clear_active(stdout);
 
-		fill = max * percent_complete() / 100;
+		if(max == total) {
+			fill = sum;
+		} else {
+			fill = max * percent_complete() / 100;
+		}
 
 		if(color_len) {
 			memset(buf, ' ', sizeof(buf));
