@@ -3,7 +3,7 @@
  * tup - A file-based build system
  *
  * Copyright (C) 2010  James McKaskill
- * Copyright (C) 2010-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2010-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -48,6 +48,10 @@ static char wintmpdir[PATH_MAX];
 
 int server_pre_init(void)
 {
+	if(conemu_injected()) {
+		fprintf(stderr, "tup error: Tup is known to not work when the ConEmu DLL (ConEmuHk.dll) is injected. Please disable ConEmuHk.dll injection or run tup outside of ConEmu. See also: https://github.com/gittup/tup/issues/419\n");
+		return -1;
+	}
 	return 0;
 }
 
@@ -439,19 +443,6 @@ int server_is_dead(void)
 	return (event_got != -1);
 }
 
-int server_config_start(struct server *s)
-{
-	/* Currently unused - this is only needed for symlink detection in fuse. */
-	if(s) {}
-	return 0;
-}
-
-int server_config_stop(struct server *s)
-{
-	if(s) {}
-	return 0;
-}
-
 int server_parser_start(struct parser_server *ps)
 {
 	ps->root_fd = tup_top_fd();
