@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2020-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -29,16 +29,22 @@ struct tent_tree {
 	RB_ENTRY(tent_tree) linkage;
 	struct tup_entry *tent;
 };
-RB_HEAD(tent_entries, tent_tree);
+
+struct tent_entries {
+	struct tent_tree *rbh_root;
+	int count;
+};
 RB_PROTOTYPE(tent_entries, tent_tree, linkage, x);
 
+#define TENT_ENTRIES_INITIALIZER {NULL, 0}
+
+void tent_tree_init(struct tent_entries *root);
 int tent_tree_add(struct tent_entries *root, struct tup_entry *tent);
 int tent_tree_add_dup(struct tent_entries *root, struct tup_entry *tent);
 struct tent_tree *tent_tree_search(struct tent_entries *root, struct tup_entry *tent);
 struct tent_tree *tent_tree_search_tupid(struct tent_entries *root, tupid_t tupid);
 int tent_tree_copy(struct tent_entries *dest, struct tent_entries *src);
 void tent_tree_remove(struct tent_entries *root, struct tup_entry *tent);
-void tent_tree_remove_count(struct tent_entries *root, struct tup_entry *tent, int *count);
 void tent_tree_rm(struct tent_entries *root, struct tent_tree *tt);
 void free_tent_tree(struct tent_entries *root);
 
