@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2009-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2009-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -29,11 +29,6 @@ struct tupid_tree {
 	tupid_t tupid;
 };
 
-struct tree_entry {
-	struct tupid_tree tnode;
-	int type;
-};
-
 RB_HEAD(tupid_entries, tupid_tree);
 RB_PROTOTYPE(tupid_entries, tupid_tree, linkage, x);
 
@@ -42,7 +37,6 @@ int tupid_tree_insert(struct tupid_entries *root, struct tupid_tree *data);
 int tupid_tree_add(struct tupid_entries *root, tupid_t tupid);
 int tupid_tree_add_dup(struct tupid_entries *root, tupid_t tupid);
 void tupid_tree_remove(struct tupid_entries *root, tupid_t tupid);
-void tupid_tree_remove_count(struct tupid_entries *root, tupid_t tupid, int *count);
 static inline void tupid_tree_rm(struct tupid_entries *root, struct tupid_tree *tt)
 {
 	RB_REMOVE(tupid_entries, root, tt);
