@@ -2,7 +2,7 @@
  *
  * tup - A file-based build system
  *
- * Copyright (C) 2008-2020  Mike Shal <marfey@gmail.com>
+ * Copyright (C) 2008-2021  Mike Shal <marfey@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -151,9 +151,6 @@ int updater(int argc, char **argv, int phase)
 	if(check_full_deps_rebuild() < 0)
 		return -1;
 
-	argc--;
-	argv++;
-
 	for(x=0; x<argc; x++) {
 		if(strcmp(argv[x], "-d") == 0) {
 			debug_enable("tup.updater");
@@ -256,13 +253,14 @@ int generate(int argc, char **argv)
 	int x;
 	int rc;
 	const char *file_open_flags = "w";
+	const char *generate_vardict_file = "tup-generate.vardict";
 #ifdef _WIN32
 	const char *example_script = "script_name.bat | script_name.sh";
 #else
 	const char *example_script = "script_name.sh";
 #endif
 
-	for(x=1; x<argc; x++) {
+	for(x=0; x<argc; x++) {
 		if(strcmp(argv[x], "--config") == 0) {
 			if(x+1 >= argc) {
 				fprintf(stderr, "--config requires a filename");
@@ -334,11 +332,11 @@ int generate(int argc, char **argv)
 	} else {
 		varfiletent = vartent;
 	}
-	if(tup_db_read_vars(tup_top_fd(), varfiletent->parent, varfiletent->name.s, vartent, NULL) < 0)
+	if(tup_db_read_vars(varfiletent->parent, varfiletent->name.s, vartent, generate_vardict_file) < 0)
 		return -1;
 
 	printf("Parsing...\n");
-	if(create_graph(&g, TUP_NODE_DIR, -1) < 0)
+	if(create_graph(&g, TUP_NODE_DIR) < 0)
 		return -1;
 	if(tup_db_select_node_by_flags(build_graph_cb, &g, TUP_FLAGS_CREATE) < 0)
 		return -1;
@@ -348,7 +346,8 @@ int generate(int argc, char **argv)
 	 * Tupfiles have already been parsed.
 	 */
 	n = TAILQ_FIRST(&g.node_list);
-	TAILQ_REMOVE(&g.node_list, n, list);
+	if(node_remove_list(&g.node_list, n) < 0)
+		return -1;
 	while(!LIST_EMPTY(&n->edges)) {
 		remove_edge(LIST_FIRST(&n->edges));
 	}
@@ -358,7 +357,8 @@ int generate(int argc, char **argv)
 		if(!n->already_used)
 			if(parse(n, &g, NULL, 0, 0, full_deps) < 0)
 				return -1;
-		TAILQ_REMOVE(&g.plist, n, list);
+		if(node_remove_list(&g.plist, n) < 0)
+			return -1;
 		while(!LIST_EMPTY(&n->incoming)) {
 			remove_edge(LIST_FIRST(&n->incoming));
 		}
@@ -383,7 +383,12 @@ int generate(int argc, char **argv)
 	} else {
 		fprintf(generate_f, "#! /bin/sh -e%s\n", verbose_script ? "x" : "");
 	}
-	if(create_graph(&g, TUP_NODE_CMD, -1) < 0)
+#ifdef _WIN32
+	fprintf(generate_f, "set %s=%s\\%s\n", TUP_VARDICT_NAME, get_tup_top(), generate_vardict_file);
+#else
+	fprintf(generate_f, "export %s=\"%s/%s\"\n", TUP_VARDICT_NAME, get_tup_top(), generate_vardict_file);
+#endif
+	if(create_graph(&g, TUP_NODE_CMD) < 0)
 		return -1;
 	if(tup_db_select_node_by_flags(build_graph_cb, &g, TUP_FLAGS_MODIFY) < 0)
 		return -1;
@@ -426,9 +431,6 @@ int todo(int argc, char **argv)
 	int rc;
 	int do_scan = 1;
 
-	argc--;
-	argv++;
-
 	for(x=0; x<argc; x++) {
 		if(strcmp(argv[x], "--no-scan") == 0) {
 			do_scan = 0;
@@ -573,12 +575,27 @@ static int delete_files(struct graph *g)
 	struct tup_entry *tent;
 	int rc = -1;
 
-	if(g->cmd_delete_count) {
+	start_progress(g->gen_delete_root.count, -1, -1);
+	while((tt = RB_ROOT(&g->gen_delete_root)) != NULL) {
+		tent = tt->tent;
+		tent_tree_rm(&g->gen_delete_root, tt);
+		if(server_is_dead())
+			goto out_err;
+		show_result(tent, 0, NULL, "rm", 1);
+		show_progress(-1, TUP_NODE_GENERATED);
+
+		if(delete_file(tent) < 0)
+			goto out_err;
+		if(tup_del_id_force(tent->tnode.tupid, TUP_NODE_GENERATED) < 0)
+			goto out_err;
+	}
+
+	if(g->cmd_delete_root.count) {
 		char buf[64];
-		snprintf(buf, sizeof(buf), "Deleting %i command%s...\n", g->cmd_delete_count, g->cmd_delete_count == 1 ? "" : "s");
+		snprintf(buf, sizeof(buf), "Deleting %i command%s...\n", g->cmd_delete_root.count, g->cmd_delete_root.count == 1 ? "" : "s");
 		buf[sizeof(buf)-1] = 0;
 		tup_show_message(buf);
-		start_progress(g->cmd_delete_count, -1, -1);
+		start_progress(g->cmd_delete_root.count, -1, -1);
 	}
 	while((tt = RB_ROOT(&g->cmd_delete_root)) != NULL) {
 		tent = tt->tent;
@@ -594,23 +611,10 @@ static int delete_files(struct graph *g)
 		show_progress(-1, TUP_NODE_GENERATED);
 	}
 
-	start_progress(g->gen_delete_count, -1, -1);
-	while((tt = RB_ROOT(&g->gen_delete_root)) != NULL) {
-		tent = tt->tent;
-		tent_tree_rm(&g->gen_delete_root, tt);
-		if(server_is_dead())
-			goto out_err;
-		show_result(tent, 0, NULL, "rm", 1);
-		show_progress(-1, TUP_NODE_GENERATED);
-
-		if(delete_file(tent) < 0)
-			goto out_err;
-		if(tup_del_id_force(tent->tnode.tupid, TUP_NODE_GENERATED) < 0)
-			goto out_err;
-	}
 	if(!RB_EMPTY(&g->save_root)) {
 		tup_show_message("Converting generated files to normal files...\n");
 	}
+	start_progress(g->save_root.count, -1, -1);
 	while((tt = RB_ROOT(&g->save_root)) != NULL) {
 		tent = tt->tent;
 		tent_tree_rm(&g->save_root, tt);
@@ -644,11 +648,11 @@ out_err:
 static int delete_in_tree(void)
 {
 	struct graph g;
-	if(create_graph(&g, TUP_NODE_FILE, -1) < 0)
+	if(create_graph(&g, TUP_NODE_FILE) < 0)
 		return -1;
-	if(tup_db_type_to_tree(&g.cmd_delete_root, &g.cmd_delete_count, TUP_NODE_CMD) < 0)
+	if(tup_db_type_to_tree(&g.cmd_delete_root, TUP_NODE_CMD) < 0)
 		return -1;
-	if(tup_db_type_to_tree(&g.gen_delete_root, &g.gen_delete_count, TUP_NODE_GENERATED) < 0)
+	if(tup_db_type_to_tree(&g.gen_delete_root, TUP_NODE_GENERATED) < 0)
 		return -1;
 	if(delete_files(&g) < 0)
 		return -1;
@@ -799,7 +803,7 @@ static int process_config_nodes(int environ_check)
 	if(tup_db_begin() < 0)
 		return -1;
 	/* Use TUP_NODE_ROOT to count everything */
-	if(create_graph(&g, TUP_NODE_ROOT, -1) < 0)
+	if(create_graph(&g, TUP_NODE_ROOT) < 0)
 		return -1;
 	if(tup_db_select_node_by_flags(build_graph_cb, &g, TUP_FLAGS_CONFIG) < 0)
 		return -1;
@@ -808,7 +812,8 @@ static int process_config_nodes(int environ_check)
 	while(!LIST_EMPTY(&g.root->edges)) {
 		remove_edge(LIST_FIRST(&g.root->edges));
 	}
-	TAILQ_REMOVE(&g.node_list, g.root, list);
+	if(node_remove_list(&g.node_list, g.root) < 0)
+		return -1;
 	remove_node(&g, g.root);
 
 	{
@@ -844,10 +849,11 @@ static int process_config_nodes(int environ_check)
 		while(!TAILQ_EMPTY(&g.plist)) {
 			struct variant *variant;
 			int rm_node = 1;
-			struct parser_server ps;
+			struct server s;
 
 			n = TAILQ_FIRST(&g.plist);
-			TAILQ_REMOVE(&g.plist, n, list);
+			if(node_remove_list(&g.plist, n) < 0)
+				return -1;
 
 			variant = variant_search(n->tent->dt);
 
@@ -922,7 +928,8 @@ static int process_config_nodes(int environ_check)
 						goto err_rollback;
 					if(tup_db_add_variant_list(n->tent->tnode.tupid) < 0)
 						goto err_rollback;
-					TAILQ_INSERT_HEAD(&g.node_list, n, list);
+					if(node_insert_head(&g.node_list, n) < 0)
+						return -1;
 					rm_node = 0;
 					show_result(n->tent, 0, NULL, "new variant", 1);
 				} else {
@@ -935,7 +942,8 @@ static int process_config_nodes(int environ_check)
 						 * (t8055).
 						 */
 						new_variants = 1;
-						TAILQ_INSERT_HEAD(&g.node_list, n, list);
+						if(node_insert_head(&g.node_list, n) < 0)
+							return -1;
 						rm_node = 0;
 						if(variant_enable(variant) < 0)
 							return -1;
@@ -943,16 +951,14 @@ static int process_config_nodes(int environ_check)
 					show_result(n->tent, 0, NULL, "updated variant", 1);
 				}
 				compat_lock_disable();
-				if(initialize_server_struct(&ps.s, n->tent) < 0)
+				if(initialize_server_struct(&s, n->tent) < 0)
 					goto err_rollback;
-				if(server_parser_start(&ps) < 0)
-					goto err_rollback;
-				rc = tup_db_read_vars(ps.root_fd, n->tent->parent, TUP_CONFIG, n->tent, variant->vardict_file);
-				if(server_parser_stop(&ps) < 0)
+				rc = tup_db_read_vars(n->tent->parent, TUP_CONFIG, n->tent, variant->vardict_file);
+				if(handle_file_dtent(ACCESS_READ, n->tent->parent, TUP_CONFIG, &s.finfo) < 0)
 					goto err_rollback;
 				if(rc < 0)
 					goto err_rollback;
-				if(add_config_files(&ps.s.finfo, n->tent, full_deps) < 0)
+				if(add_config_files(&s.finfo, n->tent, full_deps) < 0)
 					goto err_rollback;
 				compat_lock_enable();
 
@@ -1082,15 +1088,18 @@ static int rm_variant_dir_cb(void *arg, struct tup_entry *tent)
 	n = find_node(g, tent->tnode.tupid);
 	if(n) {
 		if(n->state == STATE_REMOVING) {
-			TAILQ_REMOVE(&g->removing_list, n, list);
+			if(node_remove_list(&g->removing_list, n) < 0)
+				return -1;
 		} else {
-			TAILQ_REMOVE(&g->plist, n, list);
+			if(node_remove_list(&g->plist, n) < 0)
+				return -1;
 		}
 		while(!LIST_EMPTY(&n->incoming)) {
 			remove_edge(LIST_FIRST(&n->incoming));
 		}
 		remove_node(g, n);
-		g->num_nodes--;
+		if(n->counted)
+			g->num_nodes--;
 	}
 	return 0;
 }
@@ -1099,8 +1108,10 @@ static int mark_variant_dir_for_deletion(struct graph *g, struct node *n)
 {
 	if(tup_db_select_node_by_link(build_graph_cb, g, n->tent->tnode.tupid) < 0)
 		return -1;
-	TAILQ_REMOVE(&g->plist, n, list);
-	TAILQ_INSERT_TAIL(&g->removing_list, n, list);
+	if(node_remove_list(&g->plist, n) < 0)
+		return -1;
+	if(node_insert_tail(&g->removing_list, n) < 0)
+		return -1;
 	n->state = STATE_REMOVING;
 	return 0;
 }
@@ -1233,7 +1244,7 @@ static int process_create_nodes(void)
 		old_changes = tup_db_changes();
 
 	tup_db_begin();
-	if(create_graph(&g, TUP_NODE_DIR, TUP_NODE_GHOST) < 0)
+	if(create_graph(&g, TUP_NODE_DIR) < 0)
 		return -1;
 	/* Force total_mtime to -1 so we count directory nodes rather than use
 	 * their mtimes to determine progress. In some cases we may assign an
@@ -1246,6 +1257,21 @@ static int process_create_nodes(void)
 	TAILQ_FOREACH_SAFE(n, &g.plist, list, tmp) {
 		struct variant *node_variant = tup_entry_variant(n->tent);
 
+		if(n->tent->type == TUP_NODE_GHOST) {
+			struct tent_entries root = TENT_ENTRIES_INITIALIZER;
+
+			if(tup_db_srcid_to_tree(n->tent->tnode.tupid, &root, TUP_NODE_GENERATED) < 0)
+				return -1;
+			while(!RB_EMPTY(&root)) {
+				tt = RB_MIN(tent_entries, &root);
+				tent_tree_rm(&root, tt);
+				if(tt->tent->dt != n->tent->tnode.tupid) {
+					if(tent_tree_add(&g.gen_delete_root, tt->tent) < 0)
+						return -1;
+				}
+			}
+			continue;
+		}
 		if(n->tent->type != TUP_NODE_DIR)
 			continue;
 		if(is_virtual_tent(n->tent))
@@ -1268,9 +1294,6 @@ static int process_create_nodes(void)
 						return -1;
 				}
 			}
-			if(!node_variant->enabled) {
-				g.num_nodes--;
-			}
 		} else {
 			struct tup_entry *srctent;
 			int force_removal = 0;
@@ -1323,6 +1346,33 @@ static int process_create_nodes(void)
 
 	if(build_graph(&g) < 0)
 		return -1;
+	TAILQ_FOREACH(n, &g.node_list, list) {
+		if(n->tent->type != TUP_NODE_ROOT) {
+			struct variant *node_variant = tup_entry_variant(n->tent);
+			/* Directories in an inactive variant are skipped in
+			 * create_work, and virtual tents are skipped in
+			 * parse(). Make sure these don't count towards the
+			 * number of directories that we're parsing.
+			 */
+			if(!node_variant->enabled || is_virtual_tent(n->tent)) {
+				if(n->counted)
+					g.num_nodes--;
+			}
+
+			/* Make sure we counted any nodes we were supposed to
+			 * count. For example, a variant directory that was
+			 * deleted could be a ghost that ends up in the graph,
+			 * but then is converted to a directory in
+			 * get_rel_tent() via tup_db_set_type(). So even though
+			 * we didn't match count_flags when the node was added,
+			 * we might now.
+			 */
+			if(!n->counted && n->tent->type == g.count_flags) {
+				n->counted = 1;
+				g.num_nodes++;
+			}
+		}
+	}
 	log_graph(&g, "create");
 	if(g.num_nodes) {
 		tup_main_progress("Parsing Tupfiles...\n");
@@ -1343,7 +1393,7 @@ static int process_create_nodes(void)
 	compat_lock_enable();
 
 	if(rc == 0) {
-		if(g.gen_delete_count) {
+		if(g.gen_delete_root.count) {
 			tup_main_progress("Deleting files...\n");
 		} else {
 			tup_main_progress("No files to delete.\n");
@@ -1426,9 +1476,16 @@ static int process_update_nodes(int argc, char **argv, int *num_pruned)
 	int rc = 0;
 
 	tup_db_begin();
-	if(create_graph(&g, TUP_NODE_CMD, -1) < 0)
+	if(create_graph(&g, TUP_NODE_CMD) < 0)
 		return -1;
-	if(tup_db_select_node_by_flags(build_graph_cb, &g, TUP_FLAGS_MODIFY) < 0)
+	if(tup_db_select_node_by_flags(build_graph_non_transient_cb, &g, TUP_FLAGS_MODIFY) < 0)
+		return -1;
+
+	/* Commands in the transient list have to be processed, and we have to
+	 * add any files in the transient list to the DAG to make sure they can
+	 * be removed.
+	 */
+	if(tup_db_select_node_by_flags(build_graph_transient_cb, &g, TUP_FLAGS_TRANSIENT) < 0)
 		return -1;
 	if(build_graph(&g) < 0)
 		return -1;
@@ -1476,7 +1533,7 @@ static int check_config_todo(void)
 	int stuff_todo = 0;
 
 	/* Use TUP_NODE_ROOT to count everything */
-	if(create_graph(&g, TUP_NODE_ROOT, -1) < 0)
+	if(create_graph(&g, TUP_NODE_ROOT) < 0)
 		return -1;
 	if(tup_db_select_node_by_flags(build_graph_cb, &g, TUP_FLAGS_CONFIG) < 0)
 		return -1;
@@ -1508,7 +1565,7 @@ static int check_create_todo(void)
 	int rc;
 	int stuff_todo = 0;
 
-	if(create_graph(&g, TUP_NODE_DIR, TUP_NODE_GHOST) < 0)
+	if(create_graph(&g, TUP_NODE_DIR) < 0)
 		return -1;
 	if(tup_db_select_node_by_flags(build_graph_cb, &g, TUP_FLAGS_CREATE) < 0)
 		return -1;
@@ -1539,7 +1596,7 @@ static int check_update_todo(int argc, char **argv)
 	int stuff_todo = 0;
 	int num_pruned = 0;
 
-	if(create_graph(&g, TUP_NODE_CMD, -1) < 0)
+	if(create_graph(&g, TUP_NODE_CMD) < 0)
 		return -1;
 	if(tup_db_select_node_by_flags(build_graph_cb, &g, TUP_FLAGS_MODIFY) < 0)
 		return -1;
@@ -1568,8 +1625,28 @@ static int check_update_todo(int argc, char **argv)
 	return rc;
 }
 
-static void pop_node(struct graph *g, struct node *n)
+static int unlink_node(struct node *n)
 {
+	int dfd = tup_entry_open(n->tent->parent);
+	if(dfd < 0)
+		return -1;
+	if(unlinkat(dfd, n->tent->name.s, 0) < 0) {
+		pthread_mutex_lock(&display_mutex);
+		perror("unlinkat");
+		fprintf(stderr, "tup error: Unable to unlink transient output file: %s\n", n->tent->name.s);
+		pthread_mutex_unlock(&display_mutex);
+		return -1;
+	}
+	if(close(dfd) < 0) {
+		perror("close(dfd)");
+		return -1;
+	}
+	return 0;
+}
+
+static int pop_node(struct graph *g, struct node *n)
+{
+	int has_edges = 0;
 	while(!LIST_EMPTY(&n->edges)) {
 		struct edge *e;
 		e = LIST_FIRST(&n->edges);
@@ -1577,14 +1654,44 @@ static void pop_node(struct graph *g, struct node *n)
 			/* Put the node back on the plist, and mark it as such
 			 * by changing the state to STATE_PROCESSING.
 			 */
-			TAILQ_REMOVE(&g->node_list, e->dest, list);
-			TAILQ_INSERT_HEAD(&g->plist, e->dest, list);
+			if(node_remove_list(&g->node_list, e->dest) < 0)
+				return -1;
+			if(node_insert_head(&g->plist, e->dest) < 0)
+				return -1;
 			e->dest->state = STATE_PROCESSING;
 		}
 
+		/* Flip the transient file's edges around, so all the commands
+		 * it points to now point to it. After those commands complete,
+		 * we'll revisit this node in the DAG to remove it.
+		 */
+		if(n->transient == TRANSIENT_PROCESSING) {
+			if(create_edge(e->dest, n, TUP_LINK_NORMAL) < 0)
+				return -1;
+		}
 		remove_edge(e);
+		has_edges = 1;
 	}
-	remove_node(g, n);
+	if(n->transient == TRANSIENT_PROCESSING) {
+		/* If we are waiting for other jobs to finish before removing
+		 * the transient node, put it back on node_list. Otherwise we
+		 * can remove it immediately (likely it was in the
+		 * transient_list from a previous build), and can put it into
+		 * plist.
+		 */
+		if(has_edges) {
+			if(node_insert_head(&g->node_list, n) < 0)
+				return -1;
+		} else {
+			if(node_insert_head(&g->plist, n) < 0)
+				return -1;
+		}
+		n->state = STATE_FINISHED;
+		n->transient = TRANSIENT_DELETE;
+	} else {
+		remove_node(g, n);
+	}
+	return 0;
 }
 
 /* Returns:
@@ -1646,8 +1753,10 @@ static int execute_graph(struct graph *g, int keep_going, int jobs,
 
 	root = TAILQ_FIRST(&g->node_list);
 	DEBUGP("root node: %lli\n", root->tnode.tupid);
-	TAILQ_REMOVE(&g->node_list, root, list);
-	pop_node(g, root);
+	if(node_remove_list(&g->node_list, root) < 0)
+		return -2;
+	if(pop_node(g, root) < 0)
+		return -2;
 
 	start_progress(g->num_nodes, g->total_mtime, jobs);
 	/* Keep going as long as:
@@ -1664,19 +1773,24 @@ static int execute_graph(struct graph *g, int keep_going, int jobs,
 			/* Here STATE_FINISHED means we're on the node_list,
 			 * therefore not ready for processing.
 			 */
-			TAILQ_REMOVE(&g->plist, n, list);
-			TAILQ_INSERT_HEAD(&g->node_list, n, list);
+			if(node_remove_list(&g->plist, n) < 0)
+				return -2;
+			if(node_insert_head(&g->node_list, n) < 0)
+				return -2;
 			n->state = STATE_FINISHED;
 			goto check_empties;
 		}
 
 		if(!n->expanded) {
-			TAILQ_REMOVE(&g->plist, n, list);
-			pop_node(g, n);
+			if(node_remove_list(&g->plist, n) < 0)
+				return -2;
+			if(pop_node(g, n) < 0)
+				return -2;
 			goto check_empties;
 		}
 
-		TAILQ_REMOVE(&g->plist, n, list);
+		if(node_remove_list(&g->plist, n) < 0)
+			return -2;
 		active++;
 
 		wt = LIST_FIRST(&free_list);
@@ -1713,12 +1827,14 @@ check_empties:
 			active--;
 
 			if(wt->rc == 0) {
-				pop_node(g, n);
+				if(pop_node(g, n) < 0)
+					return -2;
 			} else {
 				/* Failed jobs sit on the node_list until the
 				 * graph is destroyed.
 				 */
-				TAILQ_INSERT_TAIL(&g->node_list, n, list);
+				if(node_insert_tail(&g->node_list, n) < 0)
+					return -2;
 				failed++;
 			}
 		}
@@ -1803,7 +1919,7 @@ static void *run_thread(void *arg)
 static int create_work(struct graph *g, struct node *n)
 {
 	int rc = 0;
-	if(n->tent->type == TUP_NODE_DIR || n->tent->type == TUP_NODE_GHOST) {
+	if(n->tent->type == TUP_NODE_DIR) {
 		if(tup_entry_variant(n->tent)->enabled) {
 			if(n->already_used) {
 				rc = 0;
@@ -1817,6 +1933,7 @@ static int create_work(struct graph *g, struct node *n)
 		  n->tent->type == TUP_NODE_GENERATED ||
 		  n->tent->type == TUP_NODE_GROUP ||
 		  n->tent->type == TUP_NODE_GENERATED_DIR ||
+		  n->tent->type == TUP_NODE_GHOST ||
 		  n->tent->type == TUP_NODE_CMD) {
 		rc = 0;
 	} else {
@@ -1893,6 +2010,9 @@ static int update_work(struct graph *g, struct node *n)
 			}
 			if(tup_db_unflag_modify(n->tnode.tupid) < 0)
 				rc = -1;
+			if(is_transient_tent(n->tent))
+				if(tup_db_unflag_transient(n->tnode.tupid) < 0)
+					rc = -1;
 			pthread_mutex_unlock(&db_mutex);
 		}
 	} else {
@@ -1906,6 +2026,13 @@ static int update_work(struct graph *g, struct node *n)
 		}
 		if(tup_db_unflag_modify(n->tnode.tupid) < 0)
 			rc = -1;
+		if(n->transient == TRANSIENT_DELETE) {
+			if(unlink_node(n) < 0)
+				rc = -1;
+			if(rc == 0)
+				if(tup_db_unflag_transient(n->tnode.tupid) < 0)
+					rc = -1;
+		}
 
 		/* For environment variables, if there are no more
 		 * out-going edges, then this variable is no longer
@@ -1929,9 +2056,9 @@ static int generate_work(struct graph *g, struct node *n)
 {
 	int rc = 0;
 	char *expanded_name = NULL;
-	struct tent_entries sticky_root = {NULL};
-	struct tent_entries normal_root = {NULL};
-	struct tent_entries group_sticky_root = {NULL};
+	struct tent_entries sticky_root = TENT_ENTRIES_INITIALIZER;
+	struct tent_entries normal_root = TENT_ENTRIES_INITIALIZER;
+	struct tent_entries group_sticky_root = TENT_ENTRIES_INITIALIZER;
 	if(g) {/* unused */}
 
 	if(n->tent->type == TUP_NODE_CMD) {
@@ -1996,7 +2123,7 @@ static int move_outputs(struct node *n)
 
 	LIST_FOREACH(e, &n->edges, list) {
 		output = e->dest;
-		if(!skip_output(output->tent)) {
+		if(!skip_output(output->tent) && output->transient != TRANSIENT_DELETE) {
 			int output_dfd;
 			/* TODO: This is only required to create generated
 			 * directories. This should probably be moved
@@ -2239,7 +2366,7 @@ static int unlink_outputs(int dfd, struct node *n)
 	struct node *output;
 	LIST_FOREACH(e, &n->edges, list) {
 		output = e->dest;
-		if(!skip_output(output->tent)) {
+		if(!skip_output(output->tent) && output->transient != TRANSIENT_DELETE) {
 			int output_dfd = dfd;
 			output->skip = 0;
 			if(output->tent->dt != n->tent->dt) {
@@ -2272,6 +2399,25 @@ static int unlink_outputs(int dfd, struct node *n)
 	return 0;
 }
 
+static int mark_transient_outputs(struct node *n)
+{
+	/* Put all outputs of a transient command into the transient_list. If
+	 * the build stops due to failure or partial update, we'll be able to
+	 * revisit them on future builds to ensure they are removed when
+	 * appropriate.
+	 */
+	struct edge *e;
+	struct node *output;
+	LIST_FOREACH(e, &n->edges, list) {
+		output = e->dest;
+		if(!skip_output(output->tent)) {
+			if(tup_db_add_transient_list(output->tent->tnode.tupid) < 0)
+				return -1;
+		}
+	}
+	return 0;
+}
+
 static int process_output(struct server *s, struct node *n,
 			  struct timespan *ts,
 			  const char *expanded_name,
@@ -2406,7 +2552,7 @@ static int expand_group(FILE *f, struct estring *e, struct expand_info *info)
 		struct tup_entry *group_tent = tt->tent;
 
 		if(memcmp(group_tent->name.s, info->groupname, info->grouplen) == 0) {
-			struct tent_entries inputs = {NULL};
+			struct tent_entries inputs = TENT_ENTRIES_INITIALIZER;
 			struct tent_tree *ttinput;
 
 			if(info->used_groups_root)
@@ -2632,6 +2778,7 @@ static int update(struct node *n)
 	int compare_outputs = 0;
 	int run_in_bash = 0;
 	int use_server = 0;
+	int remove_transients = 0;
 
 	timespan_start(&ts);
 	if(n->tent->flags) {
@@ -2647,6 +2794,9 @@ static int update(struct node *n)
 				case 'b':
 					run_in_bash = 1;
 					break;
+				case 't':
+					remove_transients = 1;
+					break;
 				default:
 					pthread_mutex_lock(&display_mutex);
 					show_result(n->tent, 1, NULL, NULL, 1);
@@ -2713,6 +2863,10 @@ static int update(struct node *n)
 	pthread_mutex_lock(&db_mutex);
 	pthread_mutex_lock(&display_mutex);
 	rc = process_output(&s, n, &ts, expanded_name, compare_outputs);
+	if(rc == 0 && remove_transients) {
+		if(mark_transient_outputs(n) < 0)
+			rc = -1;
+	}
 	pthread_mutex_unlock(&display_mutex);
 	pthread_mutex_unlock(&db_mutex);
 	free(expanded_name);
