@@ -1,7 +1,7 @@
 #! /bin/sh -e
 # tup - A file-based build system
 #
-# Copyright (C) 2018-2020  Mike Shal <marfey@gmail.com>
+# Copyright (C) 2018-2021  Mike Shal <marfey@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -40,14 +40,7 @@ check_exist ignoredir/file.txt
 
 # Make sure a subsequent update doesn't do anything, since all ignored files
 # should be in the database.
-update > .tup/.tupoutput
-for i in "Tupfiles" "files" "commands"; do
-	if ! grep "No $i" .tup/.tupoutput > /dev/null; then
-		cat .tup/.tupoutput
-		echo "Error: Expected \"No $i\" in tup output" 1>&2
-		exit 1
-	fi
-done
+update_null "All ignored files should be in the database"
 
 # Make sure running with ignored files already in the db still works.
 tup touch run.sh
